@@ -1,5 +1,5 @@
 proc ButtonEditorProc {} {
-global beparent be xf bcomm bargs bindex
+global beparent be xf bcomm bargs bindex tk_version
 
 set xf(be_on) 1
 if [winfo exists .button_edit] {
@@ -17,6 +17,14 @@ ConvertHelp
 ConvertHelp
 }
 
+for {set i 0} {$i < 98} {incr i} {
+if {[string match $bargs($i) ""]} {
+set bargs($i) [list -text {} -bd 2 -relief raised -bd 2\
+-fg #000000 -bg #d9d9d9  -activebackground #ececec\
+-activeforeground Black]
+}
+}
+
 SetCnfMenuBg
 BE_UnSetTrace
 
@@ -33,9 +41,11 @@ set be(dialog) 0
 set be(win) 0
 set be(update) 0
 set be(bg) 0
+set be(quiet) 0
 
 set beparent [toplevel .button_edit  -height 150 -width 500]
 wm geometry $beparent +100+100
+wm protocol $beparent WM_DELETE_WINDOW {set be(editedi) 0;BE_Close}
 frame $beparent.entframe -relief groove -bd 3 -height 60 -width 500
 
 set bemfont "*helvetica-bold-r-*-*-12*"
@@ -44,11 +54,22 @@ menubutton $beparent.menu.toolmenu -text Tools \
 -menu $beparent.menu.toolmenu.m\
 -underline 0 -height 1 -font $bemfont
 menu $beparent.menu.toolmenu.m
-$beparent.menu.toolmenu.m add command -label "Put Button"  \
--underline 0 -command PutButton -state disabled
+$beparent.menu.toolmenu.m add command -label "Copy Button"  \
+-underline 5 -command CopyButton -state disabled
+$beparent.menu.toolmenu.m add command -label "Copy Style"  \
+-underline 0 -command CopyStyle -state disabled
+
 $beparent.menu.toolmenu.m add separator
-$beparent.menu.toolmenu.m add command -label "Write Help"  \
+$beparent.menu.toolmenu.m add command -label "Write Help..."  \
 -underline 0 -command {WriteHelp}
+$beparent.menu.toolmenu.m add separator
+$beparent.menu.toolmenu.m add command -label "Reload Buttons..."  \
+-underline 0 -command {ReRead}
+if {[set tk_version] > 4.0} {
+$beparent.menu.toolmenu.m add separator
+$beparent.menu.toolmenu.m add command -label "Send Button..."  \
+-underline 0 -command {SendButton}
+}
 message $beparent.menu.msg -text "Please, CLICK the ActionButton \
 You want to edit.." -fg "#a00000" -width 400 -anchor center \
 -justify center
@@ -66,8 +87,8 @@ frame $beparent.entframe.fc
 checkbutton $beparent.entframe.fc.chkcfr -text "Confirm" -bd 2 \
 -relief ridge\
 -onvalue "C" -offvalue "" -variable be(confirm)
-checkbutton $beparent.entframe.fc.chkdia -text "Dialog" -bd 2 -relief ridge\
--onvalue "D" -offvalue "" -variable be(dialog)
+checkbutton $beparent.entframe.fc.chkqui -text "Quiet" -bd 2 -relief ridge\
+-onvalue "Q" -offvalue "" -variable be(quiet)
 checkbutton $beparent.entframe.fc.chkwin -text "OutputWin" -bd 2 \
 -relief ridge\
 -onvalue "W" -offvalue "" -variable be(win)
@@ -102,6 +123,10 @@ $cm add radio -label Delete -variable be(cmdtxt) \
 -command [list BE_Hack {FileOps rm %s}] \
 -value Delete
 
+$cm add radio -label DirButton -variable be(cmdtxt) \
+-command [list BE_Hack {DirButton %s} pack] \
+-value DirButton
+
 $cm add radio -label DirSize -variable be(cmdtxt) \
 -command [list BE_Hack {DirSize %s}] \
 -value DirSize
@@ -138,6 +163,10 @@ $cm add radio -label Rename -variable be(cmdtxt) \
 -command [list BE_Hack {Rename %s}] \
 -value Rename
 
+$cm add radio -label VirtualLha -variable be(cmdtxt) \
+-command [list BE_Hack {VirtualLha %f %s}] \
+-value VirtualLha
+
 $cm add radio -label VirtualTar -variable be(cmdtxt) \
 -command [list BE_Hack {VirtualTar %f %s}] \
 -value VirtualTar
@@ -157,7 +186,7 @@ button $beparent.middle.advbutt -textvariable be(editmode)\
 message $beparent.entframe.up.3.label -aspect 200 \
 -text "                                               "
 
-eval {menubutton $beparent.entframe.up.3.button -textvariable $be(txt) \
+eval {menubutton $beparent.entframe.up.3.button -textvariable be(txt) \
 -width 6 -menu $beparent.entframe.up.3.button.stylemenu} $be(style)
 
 set sm [menu $beparent.entframe.up.3.button.stylemenu -tearoff 1]
@@ -187,16 +216,14 @@ $sm add radio -label Raised1 -command {ButtonStyles {-relief raised -bd 1 \
 $sm add radio -label Warning -command {ButtonStyles {-relief raised -bd 3 \
 -activebackground "#ffa0a0" -cursor pirate }}
 
-$sm add radio -label Trek -command {ButttonStyles {-relief raised -bd 3 \
+$sm add radio -label Trek -command {ButtonStyles {-relief raised -bd 3 \
 -activebackground "#ececec" -cursor trek}}
 
 frame $beparent.bottom -relief groove -bd 3 -height 50 -width 500
 
 
 frame $beparent.bottom.applyframe -relief sunken -bd 1 -height 34 -width 65
-button $beparent.bottom.applyframe.apply -text Apply -command {
-eval {ApplyProc $bindex}
-} -state disabled
+button $beparent.bottom.applyframe.apply -text Apply -command {eval {ApplyProc $bindex}} -state disabled
 button $beparent.bottom.undo -text Undo -command UndoProc  -state disabled
 
 button $beparent.bottom.save -text Save\
@@ -331,7 +358,7 @@ scale $beparent.adv.0.opt.1.bds -from 0 -to 5 -orient horizontal\
 label $beparent.adv.0.args.arglabel -text "Button arguments:"
 entry $beparent.adv.0.args.argentry -textvariable be(style) -width 24
 
-wm title $beparent "ButtonEditor v1.1 "
+wm title $beparent "ButtonEditor v1.3 "
 wm resizable $beparent 0 0
 
 pack  $beparent.menu -side top -fill x
@@ -351,7 +378,7 @@ pack $beparent.entframe.up.3.label -side top
 place $beparent.entframe.up.3.button -relx 0.5 -rely 0.5 -anchor center
 raise $beparent.entframe.up.3.button
 pack $beparent.entframe.fc.chkcfr -side left -fill x -expand true
-pack $beparent.entframe.fc.chkdia -side left -fill x -expand true
+pack $beparent.entframe.fc.chkqui -side left -fill x -expand true
 pack $beparent.entframe.fc.chkwin -side left -fill x -expand true
 pack $beparent.entframe.fc.chkupd -side right -fill x -expand true
 pack $beparent.entframe.fc -fill x -side bottom -pady 5
@@ -420,7 +447,7 @@ proc BE_Close {} {
 global xf bargs bcomm bindex be
 
 if [info exists bindex] {
-if {[set par "$be(dialog)$be(update)$be(confirm)$be(win)"] == ""} {
+if {[set par "$be(dialog)$be(update)$be(confirm)$be(win)$be(quiet)"] == ""} {
 set par N
 }
 if {$be(bg) == "&"} {
@@ -447,7 +474,7 @@ unset xf($var)
 
 set xf(editbutton) ""
 catch {unset bindex be}
-destroy .button_edit
+catch {destroy .button_edit}
 
 set xf(be_on) 0
 BE_UnSetTrace
@@ -461,7 +488,7 @@ set be(dialog) D
 } {
 set be(dialog) ""
 }
-if {[set par "$be(dialog)$be(update)$be(confirm)$be(win)"] == ""} {
+if {[set par "$be(dialog)$be(update)$be(confirm)$be(win)$be(quiet)"] == ""} {
 set par N
 }
 if {$be(bg) == "&"} {
@@ -477,11 +504,81 @@ $xf(editbutton) flash
 set be(editedi) 1
 }
 
-proc PutButton {} {
+proc ReRead {} {
+global xf
+if ![AskWin {This will DESTROY all unsaved changes!\
+Do You want to proceed ?}] {
+return
+}
+LoadConfigFile xf "xfiles.buttons"
+RefreshAll
+}
+proc RefreshAll {} {
+global bargs bcomm
+for {set y 0} {$y < 8} {incr y} {
+for {set x 0} {$x < 6} {incr x} {
+set l ".xfleft.bf([expr $y *2]).f($x).b($x)"
+set r ".xfright.bf([expr 2 * $y +1]).f($x).b($x)"
+eval {$l config } $bargs([GetButtonIndex $l])
+eval {$r config } $bargs([GetButtonIndex $r])
+}
+}
+}
+
+proc CopyStyle {} {
+global beparent bindex xf be bargs bcomm
+set cbutton $xf(editbutton)
+set cindex $bindex
+incr xf(editmode)
+. config -cursor cross
+$beparent.menu.msg config -text "Copy style to..."\
+-fg "#a00000" -width 400 -anchor center -justify center
+button $beparent.menu.cb -text Stop -bg #ffacac -padx 1 -pady 1 \
+-command {
+incr xf(editmode) -1
+set tmp $xf(editbutton)
+set xf(editbutton) ""
+}
+pack $beparent.menu.msg -side left
+pack $beparent.menu.cb -side right -ipadx 0 -ipady 0 -padx 0 -pady 0
+while 1 {
+tkwait variable xf(editbutton)
+if {$xf(editbutton) != ""} {
+set index [GetButtonIndex $xf(editbutton)]
+set i [lsearch $bargs($cindex) "-text"]
+set j [lsearch $bargs($index) "-text"]
+if {$j >= 0} {
+set text [lrange $bargs($index) $j [expr $j +1]]
+} {
+set text ""
+}
+if {$i >= 0} {
+set bargs($index)  "$text [lreplace $bargs($cindex) $i [expr $i +1]]"
+} {
+set bargs($index) "$text $bargs($cindex)"
+}
+eval {$xf(editbutton) config} $bargs($index)
+$xf(editbutton) flash
+set be(editedi) 1
+set xf(editbutton) $cbutton
+set bindex $cindex
+} {
+break
+}
+
+}
+set xf(editbutton) $cbutton
+set bindex $cindex
+pack forget $beparent.menu.msg
+catch "destroy $beparent.menu.cb"
+SetArrowPointer
+}
+
+proc CopyButton {} {
 global beparent bindex xf be bargs bcomm
 incr xf(editmode)
 . config -cursor cross
-$beparent.menu.msg config -text "Where You want to PUT it ?!?"\
+$beparent.menu.msg config -text "Copy button to ..."\
 -fg "#a00000" -width 400 -anchor center -justify center
 button $beparent.menu.cb -text Cancel -bg #ffacac -padx 1 -pady 1 \
 -command {
@@ -489,6 +586,7 @@ incr xf(editmode) -1
 set tmp $xf(editbutton)
 set xf(editbutton) ""
 set xf(editbutton) $tmp
+set so 0
 }
 pack $beparent.menu.msg -side left
 pack $beparent.menu.cb -side right -ipadx 0 -ipady 0 -padx 0 -pady 0
@@ -611,6 +709,24 @@ set be(undoargs) $bargs($bindex)
 set be(undocomm) $bcomm($bindex)
 }
 
+proc SendButton {} {
+global be bcomm bargs xf smtp bindex
+
+if [AskWin "This will mail the  current \nconfiguration of Actionbutton to 'AB-Gallery'!"] {
+set viesti "X-Files-Version: $xf(version)\nX-Files-AB-Gallery: $be(txt)\nset bargs($bindex) \{$bargs($bindex)\}\nset bcomm($bindex) \[list $bcomm($bindex)\]"
+
+if [catch {SMTP_SendMail $xf(user_email) $xf(mail) AB-Gallery $viesti} err] {
+} {
+if [info exists smtp(failure)] {
+MessageBox "MAIL NOT SEND!! (message from server: $smtp(failure))"
+} {
+MessageBox "Mail sent to AB-Gallery!!"
+}
+}
+
+}
+}
+
 proc WriteHelp {} {
 global xf
 if [winfo exists .write_help] {
